#include <time.h>

#include <chrono>
#include <thread>

// What is the point of this file and API?
//
// Well, this is necessary on embedded  devices, where you have to manually
// prepare/setup your clock for the system. Originally (for the embedded version
// of the project) this implemented with thesese functions.
//
//  - SysTick_Init() was required to setup the hardware peripheral of the
//  embedded device for counting the time. (Simple timer)
//  - SysTick_GetTime() returns time based on how the timer counted it
//  - SysTick_DelayMs() is just handy function for sleeping that used the
//  facillity provided by the timer
//
// In the desktop version this is not really necessary... unless we find out
// that getting number of milliseconds in C in multiplatform way is not as easy
// as we would like to. Given that we switched this file to C++ which has
// standard way of returning milliseconds. We chose to keep the API same as in
// case of embedded version to give you at least a little feeling of embedded.

std::chrono::system_clock::time_point start;

extern "C" {

void SysTick_Init() { start = std::chrono::system_clock::now(); }

uint32_t SysTick_GetTime() {
    auto t_now = std::chrono::system_clock::now();
    return std::chrono::duration_cast<std::chrono::milliseconds>(t_now - start)
        .count();
}

void SysTick_DelayMs(uint32_t n) {
    std::chrono::milliseconds t{n};
    std::this_thread::sleep_for(t);
}
}
